/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include "FCFS.h"

using namespace std;

void FCFS::prepare_run(int run_index)
{
    cout << endl;
    cout << "------------------------------" << endl;
    cout << "First Come First Served (FCFS): run #" << run_index << endl;
    cout << "------------------------------" << endl;
}

void FCFS::run_algorithm()
{
    int quantum = 0;

    ready_queue = move(job_queue);

    // Run all the processes up to the quantum limit.
    while ((quantum <= quantum_limit) && (ready_queue.size() > 0))
    {
        if (   (ready_queue.size() == 0)
            || (quantum < ready_queue.front()->arrival_time))
        {
            // No process to run.
            timeline += '-';
            quantum++;
        }
        else
        {
            // Run the process at the head of the ready queue.
            Process *p = ready_queue.front();
            ready_queue.erase(ready_queue.begin());

            double exp_rt = p->expected_run_time;
            p->stats.wait_time += quantum - p->arrival_time;
            end_time = quantum + exp_rt;

            // Take over the CPU until done.
            do
            {
                timeline += p->name;
                exp_rt -= 1.0;
                quantum++;
            } while (exp_rt > 0.0);

            p->stats.turnaround_time = p->stats.wait_time + p->expected_run_time;
            completed.push_back(p);
        }
    }
}
