/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include "RR.h"

using namespace std;

void RR::prepare_run(int run_index)
{
    cout << endl;
    cout << "----------------" << endl;
    cout << "Round Robin (RR): run #" << run_index << endl;
    cout << "----------------" << endl;
}

void RR::run_algorithm()
{
    int quantum = 0;

    // Nothing to do until the first process arrives.
    while (quantum < job_queue[0]->arrival_time)
    {
        timeline += '-';
        quantum++;
    }

    do
    {
        // Append any newly arrived jobs to the back of the ready queue.
        while (   (job_queue.size() > 0)
               && (quantum >= job_queue.front()->arrival_time))
        {
            Process *p = job_queue.front();
            p->stats.wait_time += quantum - p->arrival_time;
            p->start_time = quantum;
            p->previous_quantum = quantum - 1;

            ready_queue.push_back(p);
            job_queue.erase(job_queue.begin());
        }

        // Any process at the head of the ready queue runs.
        if (ready_queue.size() == 0)
        {
            timeline += '-';
        }
        else
        {
            // Run the process.
            Process *p = ready_queue.front();
            p->stats.wait_time += quantum - p->previous_quantum - 1;
            p->previous_quantum = quantum;
            timeline += p->name;

            // Process finished?
            if (p->remaining_time <= 1.0)
            {
                // Yes: Remove from the ready queue.
                ready_queue.erase(ready_queue.begin());
                completed.push_back(p);
                p->stats.turnaround_time = quantum + p->remaining_time
                                                            - p->arrival_time;
                end_time = quantum + p->remaining_time;
            }
            else
            {
                // No: Move to the back of the ready queue.
                p->remaining_time -= 1;
                ready_queue.push_back(p);
                ready_queue.erase(ready_queue.begin());
            }
        }

        quantum++;
    } while ((job_queue.size() > 0) || (ready_queue.size() > 0));
}
