/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include <algorithm>
#include "SJF.h"

using namespace std;

void SJF::prepare_run(int run_index)
{
    cout << endl;
    cout << "------------------------" << endl;
    cout << "Shortest Job First (SJF): run #" << run_index << endl;
    cout << "------------------------" << endl;
}

void SJF::run_algorithm()
{
    int quantum = 0;

    // Nothing to do until the first process arrives.
    while (quantum < job_queue[0]->arrival_time)
    {
        timeline += '-';
        quantum++;
    }

    // Run all the processes up to the quantum limit.
    do
    {
        // Append any newly arrived jobs to the back of the ready queue.
        while (   (job_queue.size() > 0)
               && (quantum >= job_queue.front()->arrival_time))
        {
            Process *p = job_queue.front();
            p->stats.wait_time += quantum - p->arrival_time;
            p->start_time = quantum;
            p->previous_quantum = quantum - 1;

            ready_queue.push_back(p);
            job_queue.erase(job_queue.begin());

            // Sort the ready queue by expected run time.
            sort(ready_queue.begin(), ready_queue.end(),
                 [] (Process *p1, Process *p2)
                    { return p1->expected_run_time < p2->expected_run_time; });
        }

        // Any process at the head of the ready queue runs.
        if (ready_queue.size() == 0)
        {
            timeline += '-';
            quantum++;
        }
        else
        {
            Process *p = ready_queue.front();
            ready_queue.erase(ready_queue.begin());

            double exp_rt = p->expected_run_time;
            p->stats.wait_time += quantum - p->arrival_time;
            end_time = quantum + exp_rt;

            // Take over the CPU until done.
            do
            {
                timeline += p->name;
                exp_rt -= 1.0;
                quantum++;
            } while (exp_rt > 0.0);

            p->stats.turnaround_time = p->stats.wait_time + p->expected_run_time;
            completed.push_back(p);
        }
    } while (quantum < quantum_limit);
}
